/*
  from_gui.cpp - Determine if a program was called from a GUI window.

  Jason Hood, 28 October, 2010.

  I would use C, but could not find a method of calling a function before main,
  so C++ it is.
*/

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <conio.h>
#include <stdio.h>


static bool from_gui_init();
static void from_gui_exit();

extern "C" int from_gui = from_gui_init();
extern "C" int from_gui_tip = 0;

static const char from_gui_exit_text[]	= "\nPress any key to exit";
static const char from_gui_tip_text[]	= " (try running from Command Prompt)";
static const char from_gui_exit1_text[] = ".";


// Simple test to see if we were run from a GUI window - check if the cursor is
// at the very start of the console buffer.
bool from_gui_init()
{
  CONSOLE_SCREEN_BUFFER_INFO csbi;
  if (GetConsoleScreenBufferInfo( GetStdHandle( STD_OUTPUT_HANDLE ), &csbi )
      && csbi.dwCursorPosition.X == 0 && csbi.dwCursorPosition.Y == 0)
  {
    atexit( from_gui_exit );
    return true;
  }
  return false;
}


// If anything was output (via the cursor moving), require a key press to exit.
void from_gui_exit()
{
  CONSOLE_SCREEN_BUFFER_INFO csbi;
  if (GetConsoleScreenBufferInfo( GetStdHandle( STD_OUTPUT_HANDLE ), &csbi )
      && !(csbi.dwCursorPosition.X == 0 && csbi.dwCursorPosition.Y == 0))
  {
    fputs( from_gui_exit_text, stdout );
    if (from_gui_tip)
      fputs( from_gui_tip_text, stdout );
    fputs( from_gui_exit1_text, stdout );
    while (_kbhit()) _getch();
    _getch();
  }
}
